/* Change the sidebar header color to match the new logo background. */
.wy-side-nav-search {
  background-color: #f6f8fa !important;
}

/* Update the sidebar background color. */
.wy-nav-side {
  background-color: #f6f8fa !important;
}

/* Update the version color to match the background color. */
select {
  color: #000 !important;
}

.version-switch {
  color: #000 !important;
}

/* Change sidebar text color to black for all sidebar items. */
.wy-nav-side .wy-nav-content,
.wy-nav-side .wy-menu-vertical a {
  color: #000 !important;
}

/* Sidebar link text color on hover. */
.wy-nav-side .wy-nav-content a:hover,
.wy-nav-side .wy-menu-vertical a:hover {
  color: #000 !important;
}

/* Sidebar item background color on hover. */
.wy-nav-side .wy-menu-vertical li.toctree-l1.current > a:hover,
.wy-nav-side .wy-menu-vertical li.toctree-l1 > a:hover {
  background-color: #f0f0eb !important;
}
/* Logo styling. */
.wy-side-nav-search .wy-nav-top img {
  padding: 15px;
  max-height: 80px;
  max-width: 100%;
  display: block;
  margin: 0 auto;
}

/* Hide HTML comments in the generated documentation. */
html > body > div.document > div.section > pre > code {
  display: none;
}
