// SPDX-License-Identifier: GPL-3.0-or-later
// This file contains the following i18n keys:
//   - "i18n._global.hello_global"
//   - "i18n._global.repeat_value_hello_global"
//   - "i18n._global.repeat_key"
//   - "i18n.repeat_value_single_file"
//   - "i18n.test_file.form_button_aria_label"
//   - "i18n.test_file.fox_image_alt_text"
//   - i18n.test_file.hello_test_file  // no quotes or backticks
//   - "i18n.test_file.incorrectly-formatted-key"
//   - "i18n.test_file.nested_example"
//   - "i18n.test_file.not_in_i18n_source_file"
//   - "i18n.test_file.repeat_key_lower"  // lower cancels the repeat value check
//   - "i18n.wrong_identifier_path.content_reference"
